/*
 * Copyright (c) 2008 Jesper Karlsson & David Gustafsson
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gtk/gtk.h>
#include <string.h>
#include <libxfce4util/libxfce4util.h>

#include "xfbib-bibtex-entry.h"
#include "xfbib-bibtex-field.h"

struct _XfbibBibtexEntry
{
	GObject parent;
	GObject *owner;
	gchar *bibtype;
	gchar separator;
	gchar *key;
	GList *fields;
};

typedef struct _XfbibBibtexEntryClass
{
	GObjectClass parent;
} XfbibBibtexEntryClass;

static void xfbib_bibtex_entry_class_init(XfbibBibtexEntryClass *klass);

static void xfbib_bibtex_entry_init(XfbibBibtexEntry *instance);
static void xfbib_bibtex_entry_finalize(GObject *obj);

G_DEFINE_TYPE(XfbibBibtexEntry, xfbib_bibtex_entry, G_TYPE_OBJECT)

static void
xfbib_bibtex_entry_class_init(XfbibBibtexEntryClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;
	object_class->finalize = xfbib_bibtex_entry_finalize;
}

static void
xfbib_bibtex_entry_init(XfbibBibtexEntry *instance)
{
	instance->key = NULL;
	instance->bibtype = NULL;
	instance->fields = NULL;
}

static void
xfbib_bibtex_entry_finalize(GObject *obj)
{
	XfbibBibtexEntry *entry = XFBIB_BIBTEX_ENTRY(obj);
	g_free(entry->key);
	g_free(entry->bibtype);
	g_list_foreach (entry->fields, (GFunc) g_object_unref, NULL);
	g_list_free (entry->fields);
	G_OBJECT_CLASS(xfbib_bibtex_entry_parent_class)->finalize(obj);
}

XfbibBibtexEntry *
xfbib_bibtex_entry_new(GObject *obj)
{
	XfbibBibtexEntry *entry;
	entry = g_object_new(XFBIB_TYPE_BIBTEX_ENTRY, NULL);
	entry->owner = obj;
	return entry;
}


GObject *
xfbib_bibtex_entry_get_owner(XfbibBibtexEntry *entry)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_ENTRY(entry), NULL);
	return entry->owner;
}
	
gboolean
xfbib_bibtex_entry_parse(XfbibBibtexEntry *entry, const gchar *str)
{
	gchar *tmp;
	GString *buf;
	XfbibBibtexField *field;
	gint i, brackets, quotes, len;

	DBG("str = %s", str);

	buf = g_string_new("");

	len = strlen(str) - 1;
	/* Dont copy the last bracket */
	tmp = g_strndup(str, len);

	/* Parse entry type */
	for (i = 1; i < len && tmp[i] != '{' && tmp[i] != '('; i++)
		buf = g_string_append_c(buf, tmp[i]);
	entry->bibtype = g_strdup(buf->str);

	if (i == len) {
		return FALSE;
	}
	
	/* Separator */
	entry->separator = tmp[i];

	buf = g_string_erase(buf, 0, -1);
	/* Parse the entry key */
	for (i++; tmp[i] != '\0' && tmp[i] != ','; i++)
		buf = g_string_append_c(buf, tmp[i]);
	entry->key = g_strdup(buf->str);

	buf = g_string_erase(buf, 0, -1);

	for (i++, brackets = 0, quotes = 0; i < len; i++) {
		if (tmp[i] == ',' && ((entry->separator == '{' && brackets == 0) ||
					(entry->separator == '"' && quotes%2 == 0))) {
			field = xfbib_bibtex_field_new(G_OBJECT(entry));
			if (!xfbib_bibtex_field_parse(field, buf->str))
				return FALSE;
			entry->fields = g_list_append(entry->fields, field);
			buf = g_string_erase(buf, 0, -1);
			continue;
		} else if (tmp[i] == '{' || tmp[i] == '(') {
			brackets++;
		} else if (tmp[i] == '}' || tmp[i] == ')') {
			brackets--;
		} else if (tmp[i] == '"') {
			quotes++;
		}
		buf = g_string_append_c(buf, tmp[i]);
	}

	/* When the last row has a comma their will be nothing here to parse */
	for (i = 0; i < buf->len; i++) {
		if(!g_ascii_isspace(buf->str[i]))
			break;
	}

	if (i < buf->len) {
		field = xfbib_bibtex_field_new(G_OBJECT(entry));
		if (!xfbib_bibtex_field_parse(field, buf->str))
			return FALSE;
		entry->fields = g_list_append(entry->fields, field);
	}
	
	g_string_free(buf, TRUE);
	g_free(tmp);

	return TRUE;
}

GList *
xfbib_bibtex_entry_get_fields(XfbibBibtexEntry *entry)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_ENTRY(entry), NULL);
	return entry->fields;
}

void
xfbib_bibtex_entry_set_fields(XfbibBibtexEntry *entry, GList *list)
{
	g_return_if_fail(XFBIB_IS_BIBTEX_ENTRY(entry));
	g_list_foreach (entry->fields, (GFunc) g_object_unref, NULL);
	g_list_free (entry->fields);

	entry->fields = list;
}

gchar *
xfbib_bibtex_entry_get_key(XfbibBibtexEntry *entry)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_ENTRY(entry), NULL);
	return entry->key;
}

void 
xfbib_bibtex_entry_set_key(XfbibBibtexEntry *entry, const gchar *key)
{
	g_return_if_fail(XFBIB_IS_BIBTEX_ENTRY(entry));
	g_free(entry->key);
	entry->key = g_strdup(key);
}

gchar *
xfbib_bibtex_entry_get_bibtype(XfbibBibtexEntry *entry)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_ENTRY(entry), NULL);
	return entry->bibtype;
}


void 
xfbib_bibtex_entry_set_bibtype(XfbibBibtexEntry *entry, const gchar *bibtype)
{
	g_return_if_fail(XFBIB_IS_BIBTEX_ENTRY(entry));
	g_free(entry->bibtype);
	entry->bibtype = g_strdup(bibtype);
}

gchar
xfbib_bibtex_entry_get_separator(XfbibBibtexEntry *entry)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_ENTRY(entry), '\0');
	return entry->separator;
}

void
xfbib_bibtex_entry_set_separator(XfbibBibtexEntry *entry, gchar new_separator)
{
		entry->separator = new_separator;
}
